package com.ym.mec.biz.dal.dao;

import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.biz.dal.entity.QuestionnaireQuestionItem;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

public interface QuestionnaireQuestionItemDao extends BaseDAO<Long, QuestionnaireQuestionItem> {


    void batchInsert(@Param("questionItemList") List<QuestionnaireQuestionItem> questionItemList,
                     @Param("questionId") Long questionId,
                     @Param("topicId") Integer topicId);

    void batchDelByQuestionIdList(@Param("questionIdList") List<Long> delQuestionnaireQuestionIdList);

    void batchUpdate(@Param("updateItemList") List<QuestionnaireQuestionItem> updateItemList);

    QuestionnaireQuestionItem findByQuestionId(Long questionId);

    void batchDel(@Param("itemIdList") List<Long> delQuestionnaireQuestionItemIdList);

    List<Map<Integer, String>> queryUserName(@Param("cooperationId") Integer cooperationId);
}